==> v_sim-3.1.2.tar.bz2: v_sim-3.1.2/src/opengl.h <==
/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2005)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#ifndef OPENGL_H
#define OPENGL_H

#include <GL/gl.h>
#include <GL/glu.h> 

#include <glib.h>

#include "visu_data.h"
#include "OSOpenGL/visu_OSInterface.h"

/***************/
/* Public part */
/***************/

/**
 * PI180:
 *
 * Value of pi / 180.
 */
#define PI180 0.017453292522

/**
 * OpenGLCamera_struct:
 * @d_red: a factor for perspective from 1. to inifnity. With one, the nose of
 *         the observer is completly set on the rendered object, and the size
 *         of the observer is neglectible compared to the size of the object.
 * @gr: a private value equal to d_red / (d_red - 1.) ;
 * @theta: the theta angle in spherical coordinates of the position of the observer ;
 * @phi: the phi angle in spherical coordinates of the position of the observer ;
 * @omega: rotation of the observer on itself ;
 * @xs: a value for translation of the viewport on x axis ;
 * @ys: a value for translation of the viewport on y axis ;
 * @gross: a value of zoom.
 * 
 * Values to define the position of the observer.
 */
struct OpenGLCamera_struct
{
  /* Perspective. */
  double d_red;
  double gr; /* equal to d_red / (d_red - 1.) */
  /* Orientation. */
  double theta, phi, omega;
  /* Position. */
  double xs, ys;
  /* Zoom. */
  double gross;
};
/**
 * OpenGLCamera:
 *
 * A short way to adress #OpenGLCamera_struct objects.
 */
typedef struct OpenGLCamera_struct OpenGLCamera;
OpenGLCamera *visuCamera;

/**
 * OpenGLBox_struct:
 * @extens : not used ;
 * @dxxs2: the square value of the norm of the x vector of the box ;
 * @dyys2: the square value of the norm of the y vector of the box ;
 * @dzzs2: the square value of the norm of the z vector of the box ;
 * @p1: the coordinates of the point (0,0,0) ;
 * @p2: the coordinates of the point (1,0,0) ;
 * @p3: the coordinates of the point (1,1,0) ;
 * @p4: the coordinates of the point (0,1,0) ;
 * @p5: the coordinates of the point (0,0,1) ;
 * @p6: the coordinates of the point (1,0,1) ;
 * @p7: the coordinates of the point (1,1,1) ;
 * @p8: the coordinates of the point (0,1,1).
 *
 * Values to describe the box where the render is done. These values are 
 * linked to a #VisuData object (the currentVisuData pointer) and valid as
 * long as this object is available. This should migrate to the #VisuData object
 * in future releases.
 */
struct OpenGLBox_struct
{
  double extens; /* makes changes in modelize, project, and numberOfFacette */
  
  double dxxs2, dyys2, dzzs2;
  double p1[3], p2[3], p3[3], p4[3], p5[3], p6[3], p7[3], p8[3];
};
/**
 * OpenGLBox:
 *
 * A short way to identify #OpenGLBox_struct object.
 */
typedef struct OpenGLBox_struct OpenGLBox;
OpenGLBox *visuBox;

/**
 * OpenGLWindow_struct:
 * @width : the width of the window ;
 * @height : the height of the window ;
 * @near : the beginning of the viewport on z axis (z for observer) ;
 * @far : the end of the viewport on z axis (z for observer) ;
 * @left : the left of the viewport on x axis ;
 * @right : the right of the viewport on x axis ;
 * @bottom : the bottom of the viewport on y axis ;
 * @top : the top of the viewport on y axis ;
 *
 * Values to describe the window where the render is done.
 */
struct OpenGLWindow_struct
{
  unsigned int width, height;
  double near, far;
  double left, right, bottom, top;
};
/**
 * OpenGLWindow:
 *
 * A short way to identify #OpenGLWindow_struct object.
 */
typedef struct OpenGLWindow_struct OpenGLWindow;
OpenGLWindow *visuWindow;

/**
 * OPENGL_OBSERVE_CONSTRAINED
 *
 * This is a key for the constrained mode in the observe session.
 * In this mode, the movements are along meridians when the mouse is
 * dragged along y axis and along parallels when the movement is along x axis.
 */
#define OPENGL_OBSERVE_CONSTRAINED 0
/**
 * OPENGL_OBSERVE_WALKER
 *
 * This is a key for the constrained mode in the observe session.
 * In this mode, the movementsare those of a walking ant on a sphere,
 * when mouse move along y axis, the ant goes strait on, when mouse is
 * dragged along x axis, the ant translates on its right or on its left.
 */
#define OPENGL_OBSERVE_WALKER 1
/**
 * openGLSet_preferedObserveMethod:
 * @method: an integer that identify the method, see #OPENGL_OBSERVE_CONSTRAINED,
 *          and #OPENGL_OBSERVE_WALKER flags.
 *
 * There are two methods to move the camera in a pick and observe sesion. These
 * two methods are described in the commentary of the keys #OPENGL_OBSERVE_CONSTRAINED
 * an d#OPENGL_OBSERVE_WALKER.
 */
void openGLSet_preferedObserveMethod(int method);
/**
 * openGLGet_preferedObserveMethod:
 *
 * There are two methods to move the camera in a pick and observe sesion. These
 * two methods are described in the commentary of the keys #OPENGL_OBSERVE_CONSTRAINED
 * an d#OPENGL_OBSERVE_WALKER.
 *
 * Returns: an integer that identify the method, see #OPENGL_OBSERVE_CONSTRAINED,
 *          and #OPENGL_OBSERVE_WALKER flags.
 */
int openGLGet_preferedObserveMethod();

/**
 * createNewObjectList:
 * @size: the requested size.
 *
 * It returns the id that can be used to link glObjectList.
 * This number is also added to the list of glObjectList to be displayed.
 * The size parameter is the number of lists that could be used by
 * the user for this glObjectList. Then the next call to this function
 * will return the last value plus the size plus one.
 *
 * Returns: an identifier used by OpenGl as a list.
 */
int createNewObjectList(int size);
/**
 * setObjectList:
 * @identifier: an OpenGl id.
 *
 * Set the specified identifier as the OBJECT list that is drawn. If
 * identifier is null then the OBJECT list is deleted. The OBJECT list
 * is designed to store all the elements.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setObjectList(int identifier);

/**
 * MASK_THETA:
 *
 * Value used in the setCameraThetaPhiOmega() method to store the tetha angle.
 */
#define MASK_THETA (1 << 1)
/**
 * MASK_PHI:
 *
 * Value used in the setCameraThetaPhiOmega() method to store the phi angle.
 */
#define MASK_PHI   (1 << 2)
/**
 * MASK_OMEGA:
 *
 * Value used in the setCameraThetaPhiOmega() method to store the omega angle.
 */
#define MASK_OMEGA   (1 << 3)
/**
 * setCameraThetaPhiOmega:
 * @valueTheta: a floatinf point value in degrees ;
 * @valuePhi: a floating point value in degrees ;
 * @valueOmega: a floating point value in degrees ;
 * @mask: to specified what values will be changed.
 *
 * Change the orientation of the camera to the specified angles.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setCameraThetaPhiOmega(float valueTheta, float valuePhi, float valueOmega, int mask);
/**
 * MASK_XS:
 *
 * Value used in the setCameraXsYs() method to store the horizontal offset.
 */
#define MASK_XS (1 << 1)
/**
 * MASK_YS:
 *
 * Value used in the setCameraXsYs() method to store the vertical offset.
 */
#define MASK_YS   (1 << 2)
/**
 * setCameraXsYs:
 * @valueX: a floatinf point value in the bounding box scale
 *          (1 is the size of the bounding box) ;
 * @valueY: a floating point value in bounding box scale ;
 * @mask: to specified what values will be changed.
 *
 * Change the point where the camera is pointed to.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setCameraXsYs(float valueX, float valueY, int mask);
/**
 * setCameraGross:
 * @value: a positive floating point value.
 *
 * Change the value of the camera zoom value. If the value is higher than 10
 * it is set to 10 and if the value is negative it is set to 0.001.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setCameraGross(float value);
/**
 * setCameraPersp:
 * @value: a floating point value greater than 1.1.
 *
 * Change the value of the camera perspective value and put it in
 * bounds if needed.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setCameraPersp(float value);
/**
 * setOpenGLContext:
 * @contextIsPixmap: TRUE to render in a pixmap, FALSE to render to a window.
 * @dumpData: a pointer to store a pointer to a newly created #DumpImage.
 * 
 * Use this method to change of OpenGL context. If the context is a pixmap,
 * *dumpData points to a newly created description of this pixmap.
 *
 * Returns: 0 if no error occured. 
 */
int setOpenGLContext(gboolean contextIsPixmap, DumpImage **dumpData);

/**
 * setOpenGlMaterial:
 * @material: a 5 elements array with the material properties ;
 * @rgba: a 4 elements array with the color values.
 *
 * This method call glMaterial to create the right shiningness, emission, diffuse...
 */
void setOpenGlMaterial(float* material, float* rgba);


/* Object to deal with quadrics. */
GLUquadricObj *obj;

/**
 * initOpengl:
 * @width: a positive integer value ;
 * @height: a positive integer value ;
 *
 * This initialises the drawing window and all the variables used
 * by this module. It must be called once by the main program and should
 * not be used otherwise.
 */
int initOpengl(int width, int height);

/**
 * VisuPick_struct:
 * @node: a pointer to a #VisuNode that has been selected ;
 * @button: an integer identifying the button that clicked on the node ;
 * @shiftMod: a boolean to set if shift button was pushed during the click ;
 * @x: x coordinate of the click in window pixels ;
 * @y: y coordinate of the click in window pixels
 *
 * This structure is used when the user uses the pick
 * interface. It stores some informations on the pick action.
 */
struct VisuPick_struct
{
  VisuNode *node;
  unsigned int button;
  int shiftMod;
  int x, y;
};
/**
 * VisuPick
 *
 * A short definition for #VisuPick_struct objects.
 */
typedef struct VisuPick_struct VisuPick;
/**
 * callBackOnNodeSelectedFunc:
 * @pickData: the structure containing the informations of the
 *            pick (mouse click, position...).
 *
 * This interface defines functions that are called whenever a
 * node is selected.
 */
typedef void (*callBackOnNodeSelectedFunc) (const VisuPick *pickData);

/* observe and pick functions. */
/**
 * initPickObserve:
 * @handler: a pointer to a method to be called whenever
 *           an element is selected (can be NULL) ;
 * @stopPickObserve: a pointer to a method that will be called
 *                   every time a pick or an observe session are
 *                   finish (can be NULL).
 *
 * Initialises a period when the client can control an observe or
 * a pick session. It changes cursors shapes and initialises different
 * internal variables. It can be stop by a call to endPickObserve().
 */
void initPickObserve(callBackOnNodeSelectedFunc handler,
		     GDestroyNotify stopPickObserve);
/**
 * endPickObserve:
 *
 * Stop the period when pick or observe sessions are allowed. Basically,
 * for the user point of view it returns the cursor to its initial shape.
 */
void endPickObserve();
/**
 * beginObserve:
 * 
 * During a pick or observe session, the client can begin an observe session
 * by a call to this method. It actually stops also a possibly running
 * pick session.
 */
void beginObserve();
/**
 * beginPick:
 * 
 * During a pick or observe session, the client can begin a pick session
 * by a call to this method. It actually stops also a possibly running
 * observe session.
 */
void beginPick();

/* Draw methods */
/**
 * MARK_SQUARE_BIG
 *
 * Use this as the first argument of the drawEraseMark() method to draw
 * big squares.
 */
#define MARK_SQUARE_BIG 0
/**
 * MARK_SQUARE_SMALL
 *
 * Use this as the first argument of the drawEraseMark() method to draw
 * small squares.
 */
#define MARK_SQUARE_SMALL 1
/**
 * drawEraseMark:
 * @which: use #MARK_SQUARE_BIG or #MARK_SQUARE_SMALL ;
 * @xx: the position to draw the center of the mark, horizontal coordinate ;
 * @yy: the position to draw the center of the mark, vertical coordinate ;
 * @draw_it: a boolean to draw (1) or erase (0) the mark.
 *
 * Draw a mark in inverse color at the specified position or erase it.
 * There are different kinds of mark such as #MARK_SQUARE_BIG or #MARK_SQUARE_SMALL.
 */
void drawEraseMark(int which, int xx, int yy, int draw_it);

/**
 * dumpViewToImage:
 * @buffer: a #GString to store messages.
 *
 * This method takes a screenshot of the current content of the
 * rendering window and put it in a big array. The #GString buffer
 * is used to store messages about this process.
 *
 * Returns : a big array with all the colors of the pixels of
 * the rendering window. It returns a NULL pointer if something goes wrong.
 */
unsigned char * dumpViewToImage(GString *buffer);

/**
 * setNewWindowSize:
 * @width: the new horizontal size ;
 * @height: the new vertical size.
 *
 * It changes the size of the OpenGl area. Warning : it doesn't change
 * the size of the window.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setNewWindowSize(unsigned int width, unsigned int height);

/**
 * getNumberOfFacettes:
 * @dimension: the size of the object which asks for its number of facettes.
 *
 * This is a function to get the number of "facettes" advised
 * by the server (according to its policy on rendering)
 * to draw an object according to a given dimension.
 *
 * Returns: the number of facettes the object should used.
 */
int getNumberOfFacettes(float dimension);

/**
 * setPrecision:
 * @value: a positive value (100 is normal precision).
 *
 * This function change the value of the parameter precisionOfRendering. It
 * changes the number of facettes advised for every objects. It allows to
 * increase or decrease the number of polygons drawn and thus acts on the
 * speed of rendering.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setPrecision(float value);
/**
 * getPrecision:
 *
 * This function retrieve the value of the parameter precisionOfRendering.
 *
 * Returns: the actual precision.
 */
float getPrecision();

/**
 * setRenderingOption:
 * @value: an integer to represent the method of rendering.
 *
 * This function change the value of the parameter renderingOption.
 * It controls how V_Sim renders objects, in wireframe for example.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setRenderingOption(int value);
/**
 * setRenderingOptionFromString:
 * @value: the name of the method to render.
 *
 * This function change the value of the parameter renderingOption.
 * It uses the name of the rendering method to choose it. All the available
 * names can be retrieve by a call to getRenderingOptionAsList().
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setRenderingOptionFromString(char* value);
/**
 * getRenderingOption:
 *
 * This function retrieve the value of the parameter renderingOption.
 *
 * Returns: the identifier of the current rendering option.
 */
int getRenderingOption();
/**
 * getRenderingOptionAsString:
 *
 * This function retrieve the value of the parameter renderingOption.
 *
 * Returns: the name of the current rendering option.
 */
char* getRenderingOptionAsString();
/**
 * getRenderingOptionAsList:
 *
 * This method is useful to grep all the available names for rendering option.
 *
 * Returns: a GList with all the name of the available options for rendering.
 */
GList* getRenderingOptionAsList();

/**
 * setAntialias:
 * @value: a boolean to activate or not the lines antialias.
 *
 * To set the antialiasing on lines.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setAntialias(int value);
/**
 * getAntialias:
 *
 * Get the value of the antialiasing parameter.
 *
 * Returns: wether or not the antialising for lines is activated.
 */
int getAntialias();

/**
 * setFakeBackingStore:
 * @bool: a boolean to activate or not the faked backing-store.
 *
 * The fake backing-store is a way to act as if there was a backing-store
 * in the Xserver. It calls a redraw each time the window receive an exposure
 * event. It is not optimized at all and can be very slow. Use it only
 * if the backing-store of the Xserver is broken or not activated.
 */
void setFakeBackingStore(int bool);
/**
 * getFakeBackingStore:
 * 
 * Get the value of the fakeBackingStore option.
 *
 * Returns: wether the faked backing-store is activated or not.
 */
int getFakeBackingStore();

/**
 * setImmediateDrawing:
 * @bool: a boolean to set or not the immediateDrawing option.
 *
 * If true all changes are applied only when the refresh button
 * is pushed.
 */
void setImmediateDrawing(int bool);
/**
 * getImmediateDrawing:
 *
 * Get the value of the immediateDrawing option.
 *
 * Returns: the value of the immediateDrawing option.
 */
int getImmediateDrawing();

/****************/
/* Private part */
/****************/


/* Variables to store the glObjectList. */
/* The default size of the table to store the user
   lists. */
#define GLOBJECTLIST_INCREMENT 15
/* First value given to user to store their glObjectList. */
#define GLOBJECTLIST_FIRST 1001
/* The table that stores the identifier of user's glObjectList. */
int *glObjectListRegistered;
/* The allocated size of this table. */
int glObjectListRegisteredSize;
/* The number of stored elements. */
int nbGlObjectListRegistered;
/* The number of user's reserved numbers of the last
   entry in this table. */
int lastGlObjectListSize;


/* This variable stores the fact that OBJECT is
   allocated or not. No redraw is possible if OBJECT
   is not allocated. */
int isOBJECTReady;

enum renderingOptions
  {
    Wireframe,
    Flat,
    Smooth
  };

#define FLAG_RESOURCE_OPENGL_ANGLES "opengl_theta_phi_omega"
#define DESC_RESOURCE_OPENGL_ANGLES "2 real values (degrees) for user orientation with respect to sample"
gboolean readOpenGLThetaPhi(gchar **lines, int nbLines,
			    int position, GString *errorMessage);
#define FLAG_RESOURCE_OPENGL_TRANSLAT "opengl_xs_ys"
#define DESC_RESOURCE_OPENGL_TRANSLAT "2 real values for image position with respect to [0.0, 1.0]x[0.0, 1.0] window"
gboolean readOpenGLXsYs(gchar **lines, int nbLines,
			int position, GString *errorMessage);
#define FLAG_RESOURCE_OPENGL_GROSS "opengl_gross"
#define DESC_RESOURCE_OPENGL_GROSS "gross factor (must be real > 0.0)"
#define RESOURCE_OPENGL_GROSS_DEFAULT 1.
gboolean readOpenGLGross(gchar **lines, int nbLines,
			 int position, GString *errorMessage);
#define FLAG_RESOURCE_OPENGL_PERSP "opengl_d_red"
#define DESC_RESOURCE_OPENGL_PERSP "reduced perspective distance (must be real > 1.0)"
#define RESOURCE_OPENGL_PERSP_DEFAULT 5.
gboolean readOpenGLPersp(gchar **lines, int nbLines,
			 int position, GString *errorMessage);


gboolean exportParametersOpenGL(GString *data, int *nbLinesWritten);
gboolean exportResourcesOpenGL(GString *data, int *nbLinesWritten);

void rename_window(char *nom);

void gl_new_data(char *fn);

void project(void);

/* This callback method call the gl_new_data. */
void newFileLoaded(GObject *obj, gpointer data);

/* redraw methods */
void reDraw(GObject *obj, gpointer data);
void drawToEmpty(void);

/* this method is called by the pick routine to know
   which VisuNode is selected. */
VisuNode* getSelectElement(int x, int y);



typedef struct light_
{  
  gboolean enabled;
  float ambient[4];
  float diffuse[4];
  float specular[4];
  float position[4];
} light;

light* light_new_default();
void light_add(light *light0);

GList* light_get_list();

#endif

==> v_sim-3.2.1.tar.bz2: v_sim-3.2.1/src/opengl.h <==
/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2006)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2006)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/
#ifndef OPENGL_H
#define OPENGL_H

#include <GL/gl.h>
#include <GL/glu.h> 

#include <glib.h>

#include "visu_data.h"
#include "OSOpenGL/visu_OSInterface.h"
#include "openGLFunctions/light.h"
#include "openGLFunctions/view.h"

/***************/
/* Public part */
/***************/

/**
 * setOpenGLContext:
 * @contextIsPixmap: TRUE to render in a pixmap, FALSE to render to a window ;
 * @dumpData: a pointer to store a pointer to a newly created #DumpImage ;
 * @width: new width ;
 * @height: new height.
 * 
 * Use this method to change of OpenGL context. If the context is a pixmap,
 * dumpData points to a newly created description of this pixmap. The given
 * size (@width, @height) is used to set the size of the pixmap if the context
 * is indirect rendering.
 *
 * Returns: 0 if no error occured. 
 */
int setOpenGLContext(gboolean contextIsPixmap, DumpImage **dumpData,
		     guint width, guint height);

/**
 * openGLInit_context:
 *
 * Basic method to be called whenever a new context is used. It sets the
 * OpenGL variable values (enabling depth buffer...), applying light model and
 * rendering mode.
 */
void openGLInit_context();

/**
 * setOpenGlMaterial:
 * @material: a 5 elements array with the material properties ;
 * @rgba: a 4 elements array with the color values.
 *
 * This method call glMaterial to create the right shiningness, emission, diffuse...
 */
void setOpenGlMaterial(float* material, float* rgba);


/* Object to deal with quadrics. */
GLUquadricObj *obj;

/**
 * initOpengl:
 *
 * This initialises the drawing window and all the variables used
 * by this module. It must be called once by the main program and should
 * not be used otherwise.
 *
 * Returns: 1 if everything goes right.
 */
int initOpengl();

/* Objects to play with lights. */
LightEnvironnement *currentLights;


/* Draw methods */

/**
 * dumpViewToImage:
 * @dataObj: the #VisuData object to dump ;
 * @buffer: a #GString to store messages ;
 * @width: an integer used as I/O;
 * @height: an integer used as I/O.
 *
 * This method takes a screenshot of the current content of the
 * rendering window and put it in a big array. The #GString buffer
 * is used to store messages about this process. If width or height are
 * null or negative values, the current window size is used ; but if they
 * are positive, it change the size of the rendering area during the export.
 * The @width and @height values are modified by this method and the size
 * the dumped image is returned.
 *
 * Returns : a big array with all the colors of the pixels of
 * the rendering window. It returns a NULL pointer if something goes wrong.
 */
unsigned char * dumpViewToImage(VisuData *dataObj, GString *buffer, int *width, int *height);

/**
 * setAntialias:
 * @value: a boolean to activate or not the lines antialias.
 *
 * To set the antialiasing on lines.
 *
 * Returns: TRUE if the signal OpenGLAskForReDraw should be emitted.
 */
int setAntialias(int value);
/**
 * getAntialias:
 *
 * Get the value of the antialiasing parameter.
 *
 * Returns: wether or not the antialising for lines is activated.
 */
int getAntialias();

/**
 * setFakeBackingStore:
 * @bool: a boolean to activate or not the faked backing-store.
 *
 * The fake backing-store is a way to act as if there was a backing-store
 * in the Xserver. It calls a redraw each time the window receive an exposure
 * event. It is not optimized at all and can be very slow. Use it only
 * if the backing-store of the Xserver is broken or not activated.
 */
void setFakeBackingStore(int bool);
/**
 * getFakeBackingStore:
 * 
 * Get the value of the fakeBackingStore option.
 *
 * Returns: wether the faked backing-store is activated or not.
 */
int getFakeBackingStore();

/**
 * setImmediateDrawing:
 * @bool: a boolean to set or not the immediateDrawing option.
 *
 * If true all changes are applied only when the refresh button
 * is pushed.
 */
void setImmediateDrawing(int bool);
/**
 * getImmediateDrawing:
 *
 * Get the value of the immediateDrawing option.
 *
 * Returns: the value of the immediateDrawing option.
 */
int getImmediateDrawing();

/****************/
/* Private part */
/****************/


/* This variable stores the fact that OBJECT is
   allocated or not. No redraw is possible if OBJECT
   is not allocated. */
int isOBJECTReady;


gboolean exportParametersOpenGL(GString *data, int *nbLinesWritten);


void project(void);


/* redraw methods */
void reDraw(GObject *obj, gpointer data);
void drawToEmpty(void);



#endif
